#include "phylo-stability.h"
FILE *fopene(char *filename, char *rw, char *err){
  FILE *fp;
  if((fp=fopen(filename,rw))==NULL){
    printf("%s\n",err);
    exit(1);
  }
  return(fp);
}
char get_param(int *k, char **arg, char *value){ 
  char opt='z';
  
  *value='\0';
  if(*k==0) return('z');
  if(arg[(*k)][0]!='-'){ /* not an option, rather a value */
    sprintf(value,"%s",arg[(*k)]);
    *k = (*k)-1;
    if(arg[(*k)][0]=='-') opt=arg[(*k)][1];
  }
  else{ /* is an option */
    opt=arg[(*k)][1];
    if(arg[(*k)][1]!='\0' && arg[(*k)][2]!='\0') sprintf(value,"%s",&arg[(*k)][2]);
  }
  *k = (*k)-1;
  return(opt);
}
int names_infile(FILE *infile, char (**names)[11], int sequential){
  /* Will often work with sequential=0 even if file actually is sequential
   * If !sequential
   *   lines 2 through ntaxa+1 start with taxa names 
   *  PHYLIP format allows
   *  Archaeopt
   *  0011001101
   * if sequential option given 
   *
   * names allocated as a contiguous block */
  char c;
  int i,j,nchar,nsite,ntaxa,fso;

  fso=fscanf(infile,"%i %i",&ntaxa,&nsite);
  if(fso<2){printf("sequence file should start with ntaxa nsite\n"); exit(1);}
  *names=(char (*)[11]) malloc((size_t) ntaxa*sizeof(**names));
  while((c=getc(infile)) !='\n') ;
  if (!sequential)
    for(j=0; j<ntaxa; j++){
      for(i=0; i<10; i++)
	(*names)[j][i]=getc(infile);
      (*names)[j][10]='\0';
      while((c=getc(infile)) !='\n' && c != EOF) ;
    }
  else{
    for(j=0; j<ntaxa; j++){
      for(i=0; i<10; i++) (*names)[j][i]=getc(infile);
      nchar=0;
      while((c=getc(infile)) != ' ' && c != '\t' && c != '\r' && c != '\n'){
	nchar++;
	if (nchar == nsite) break;
      }
      while((c=getc(infile)) !='\n') ;
    } /* end of for(j=0... */
  } /* end of else */
  return(ntaxa);
}
void read_treecsnl(FILE *treefile, int ntaxa, char names[][11], double *utreec, 
		   int has_names){
  char *tstring;
  /* char tstring[10000]; */
  tstring = (char *) malloc(100000*sizeof(char));
  read_tree(treefile,tstring);
  /* printf("%s\n",tstring); */
  treecsnl(ntaxa,tstring,names,utreec,has_names);
  free(tstring);
}
void treecsnl(int ntaxa, char *tstr, char names[][11], double *utreec, 
	      int has_names){
  char **ilabels; 
  int j;
  ilabels=treecs(ntaxa, tstr, names, utreec, has_names);
  for(j=0; j<ntaxa-2; j++)free(ilabels[j]);
  free(ilabels);
}
int read_tree(FILE *fp, char *tstring)
{
  /* assumes 
   *   '[' and ']' delimit comments only
   *    '][' won't be used to end a comment and start a new one
   *   white space can be removed */
  int ltstring;
  char c;

  ltstring = 0;
  while((c = getc(fp)) != ';' && c!=EOF){ 
    if (c != ' ' && c != '\t' && c != '\r' && c != '\n'){ /* no white space */
      if (c == '[') /* start of comment; ignored */
	while((c = getc(fp)) != ']') ;
      else
	tstring[ltstring++]=c;
    }
  } 
  if(c==EOF){return(-1);}
  tstring[ltstring++]=';';
  tstring[ltstring++]='\0';
  return(ltstring);
}
int ntrees_treefile(FILE *treefile){
  int ntrees;
  char c;
  ntrees=0;
  while((c=getc(treefile)) != EOF)
    if(c==';') ntrees++;
  rewind(treefile);
  return(ntrees);
}
char **treecs(int ntaxa, char *tstr, char names[][11], double *utreec, 
	      int has_names){
  /* IN:
   * tstr - Newick tree w/o comments, whitespace and ; (see read_tree)
   * has_names = 1 if names are available
   * IN/OUT:
   * names - if has_names, used to determine ordering 0,...,ntaxa-1 of 
   *         terminal labels
   *         o/w gives names of the taxa labeled 0,...,ntaxa-1 in utreec
   * OUT:
   * utreec 
   * RETURN:
   * ilabels - internal nodel labels; (ntaxa-2); might be uninitialized
   *    not allocated as a contiguous block
   *
   * COMMENTS:
   * Based on observation that a root subtree is either
   *    (a) a leaf and has no `(' or ')' 
   * or (b) a dlist + label and el, and has equal numbers of `(' and ')' 
   *
   * violations of Newick standard
   * - quoted labels not allowed
   * - '[' and ']' only used to delimit comments
   * - limits of 10 non-null chars on leaf names
   * - underscores are not converted to blanks
   */

  int *ds; /* starting char positions of dlists */
  int *de; /* ending char positions of dlists */
  int cd=0,td=1; /* current and total dlists */
  int cj; /* current join */
  int cil; /* current internal label */
  int ctl=0; /* current terminal label */
  int *ul; /* utreec labels */
  int *ls,*le,*is_leaf,k,i,ss,K,j,ssn,ir,il;
  char **ilabels,tname[11];
  double *el; 

  el = (double *) malloc((size_t) ntaxa*sizeof(double));
  ds = (int *) malloc((size_t) 2*ntaxa*sizeof(int));
  de = (int *) malloc((size_t) 2*ntaxa*sizeof(int));
  ls = (int *) malloc((size_t) ntaxa*sizeof(int));
  le = (int *) malloc((size_t) ntaxa*sizeof(int));
  is_leaf = (int *) malloc((size_t) ntaxa*sizeof(int));
  ul = (int *) malloc((size_t) ntaxa*sizeof(int));
  ilabels = (char **)malloc((size_t) (ntaxa-2)*sizeof(char *));

  /* if(has_names) */
  /*   for(i = 0; i < ntaxa; i++) underscore2blank(names[i]); */

  for(i = 0; i < ntaxa; i++) el[i]=0;
  
  *ds=0; *de=0; *is_leaf=0;
  while(tstr[*de] != '\0')(*de)++;
  while(tstr[*de] != ')') (*de)--; /* root label and edge ignored */

  cj=ntaxa-2;
  cil=2*ntaxa-3;
  while (cj >= 0){/* Main loop */

    /* obtain all root subtrees of the current dlist and update dlists */
    ss=ds[cd]+1;
    K=0;
    while(ss <= de[cd]-1){
      el[K]=get_dlist(tstr,ss,de[cd],&ssn,&ls[K],&le[K]);
      is_leaf[K]=(ls[K] == ss)?1:0;
      if (!is_leaf[K]){
	ds[td]=ss;
	de[td]=ls[K]-1;
	td++;
      }
      ss=ssn;
      K++;
    }

    /* attach ulabels to subtrees */
    for(i = 0; i < K-2; i++){
      ilabels[cil-ntaxa]=(char *)malloc(sizeof(char));
      ilabels[cil-ntaxa][0]='\0';
      cil--;
    }
    for(i=0; i < K; i++){
      if (!is_leaf[i]){
	ilabels[cil-ntaxa]=(char *)malloc((le[i]-ls[i]+2)*sizeof(char));
	memcpy(ilabels[cil-ntaxa],&tstr[ls[i]],(le[i]-ls[i]+1)*sizeof(char));
	ilabels[cil-ntaxa][le[i]+1-ls[i]]='\0';
	/* underscore2blank(ilabels[cil-ntaxa]); */
	ul[i]=cil;
	cil--;
      }
      if (is_leaf[i] && !has_names){
	memcpy(names[ctl],&tstr[ls[i]],(le[i]-ls[i]+1)*sizeof(char));
	for(k=le[i]+1-ls[i]; k < 10; k++) names[ctl][k]=' ';
	names[ctl][10]='\0';
	/* underscore2blank(names[ctl]); */
	ul[i]=ctl;
	ctl++;
      }
      if (is_leaf[i] && has_names){
	memcpy(tname,&tstr[ls[i]],(le[i]-ls[i]+1)*sizeof(char));
	for(k=le[i]+1-ls[i]; k < 10; k++) tname[k]=' ';
	tname[10]='\0';
	/* underscore2blank(tname); */
	for(j = 0; j < ntaxa; j++){
	  if (strncmp(tname,names[j],11)==0) break;
	}
	if(j >= ntaxa){
	  printf("treecs: unable to find %s among taxa\n",tname); exit(0);}
	ul[i]=j;
      } 
    }/* end of attach ulabels ... */
    
    /* printf("current subtree %i\n",ntaxa+cj); */
    /* for(j = ds[cd]; j <= de[cd]; j++) putchar(tstr[j]); printf("\n"); */
    /* printf("number of subtrees: %i\n",K); */
    /* for(i=0; i < K; i++){ */
    /*   printf("%i %f ",ul[i],el[i]); */
    /*   if(!is_leaf[i]){ */
    /* 	for(j = ds[tdo]; j <= de[tdo]; j++) putchar(tstr[j]); */
    /* 	tdo++; */
    /*   } */
    /*   printf("\n"); */
    /* } */
    /* tdo=td; */

    /* adjust labels of existing internal edges for multifurcation */
    if(K>2)
      for(i=cj+1; i<ntaxa-1; i++)
	for(k=0; k<2; k++) 
	  if(utreec[k+i*4]>ntaxa && utreec[k+i*4]<ntaxa+cj) utreec[k+i*4] -= K-2;

    /* add root subtrees to utreec */
    for(i = 0; i < K-2; i++){
      utreec[cj*4]=ul[i]; utreec[1+cj*4]=cj-1+ntaxa; 
      utreec[2+cj*4]=el[i]; utreec[3+cj*4]=0.0;
      cj--;
    }
    if (ul[K-2] < ul[K-1]){
      ir=K-2;il=K-1;
    }
    else{
      ir=K-1;il=K-2;
    }
    utreec[cj*4]=ul[ir]; utreec[2+cj*4]=el[ir];
    utreec[1+cj*4]=ul[il]; utreec[3+cj*4]=el[il];
    cj--;
    cd++;
    
    /* for(i=cj+1; i<ntaxa-1; i++) */
    /*   printf("%i %i %i %f %f\n",i+ntaxa,(int) utreec[i*4],(int) utreec[1+i*4], */
    /* 	     utreec[2+i*4],utreec[3+i*4]); */

  } /* end of Main loop */
  /* pr_utreec(stdout,ntaxa,utreec); */
  /* for(i = 0; i < ntaxa; i++) */
  /*   printf("%s\n",names[i]); */
  
  free(is_leaf); 
  free(ul); 
  free(le); 
  free(ls); 
  free(de); 
  free(ds); 
  free(el); 
  return(ilabels);
}
double get_dlist(char *tstr, int ss, int de, int *ssn, int *ls, int *le)
{
  /* IN:
   * tstr - tree
   * ss - char position to start search
   * de - end position of dlist to be searched
   * OUT:
   * ssn - char position to start new search
   * ls - start pos of label; (ls-1) gives end of dlist found
   * le - end pos of label; 
   * RETURN: edge length (0.1 if not present) */

  char c,els[100]; /* note the limit on edge-length string */
  int lb=0,rb=0,lrb,i,j=0;
  double el=0.1;

  /* printf("%s\n",tstr); */
  /* printf("%i %i\n",ss,de); */
  /* for(i=ss; i<de; i++) putchar(tstr[i]); printf("\n"); */

  i=ss;
  lrb=ss-1;
  while(i <= de-1){
    if ((c=tstr[i]) == '(') lb++;
    if (c == ')'){rb++; lrb=i;}
    if (c == ',' && lb==rb) break;
    i++;
  }
  *ssn=i+1;
  *ls=lrb+1;
  i=lrb+1; 
  while((c=tstr[i]) != ':' && c != ','&& c != ')') i++;
  *le=i-1;
  if(c == ':'){
    while((c=tstr[++i]) != ',' && c != ')') els[j++]=c;
    els[j]='\0';
    el=atof(els);
  }
  return(el);
}
void rmexspce(int numsp, char names[][11]){
  int i,j;
  for(i = 0; i < numsp; i++)
    for(j = 9; j >= 0; j--)
      if (names[i][j] != ' ' && names[i][j] != '\0' &&  names[i][j] != '\t' && names[i][j] != '\r'){
	names[i][j+1] = '\0';
	break;
      }
}
int ndig_intpart(double f){
  int d=1,ndigits=0;
  while(((int) f/d) > 0){
    d *= 10;
    ndigits++;
  }
  if (ndigits == 0) ndigits=1;
  return(ndigits);
}
char *ut2in(int numsp, double *utreec, char names[][11], char **ilabels){
  char **tstring,*tstr;
  int i,alen,len,r,l;
  double x;

  tstring = (char **) malloc((size_t) (2*numsp-2)*sizeof(char *));

  rmexspce(numsp,names);
  for(i = 0; i < numsp; i++){
    len = strlen(names[i])+1;
    tstring[i] = (char *) malloc((size_t) len*sizeof(char));
    strcpy(tstring[i], names[i]);
  }

  for(i = 0; i < (numsp-3); i++){
    /* memory allocation */
    r = utreec[i*4]; l = utreec[1+i*4];
    len = strlen(tstring[r]) + strlen(tstring[l]) +
      ndig_intpart(utreec[2+i*4]) + ndig_intpart(utreec[3+i*4]) + 18;
    if (r >= numsp) len += strlen(ilabels[r-numsp]);
    if (l >= numsp) len += strlen(ilabels[l-numsp]);
    tstring[i+numsp] = (char *) malloc((size_t) len*sizeof(char));
    /* create subtree */
    if (r >= numsp && l >= numsp)
      alen=sprintf(tstring[i+numsp],"(%s%s:%.5f,%s%s:%.5f)",tstring[r],
	      ilabels[r-numsp],utreec[2+i*4],tstring[l],ilabels[l-numsp],
	      utreec[3+i*4]);
    if (r >= numsp && l < numsp)
      alen=sprintf(tstring[i+numsp],"(%s%s:%.5f,%s:%.5f)",tstring[r],
	      ilabels[r-numsp],utreec[2+i*4],tstring[l],utreec[3+i*4]);
    if (r < numsp && l >= numsp)
      alen=sprintf(tstring[i+numsp],"(%s:%.5f,%s%s:%.5f)",tstring[r],
	     utreec[2+i*4],tstring[l],ilabels[l-numsp],utreec[3+i*4]);
    if (r < numsp && l < numsp)
      alen=sprintf(tstring[i+numsp],"(%s:%.5f,%s:%.5f)",tstring[r],
	      utreec[2+i*4],tstring[l],utreec[3+i*4]);
    /* printf("%i %i: %s\n",alen+1,i+numsp,tstring[i+numsp]); */
  }
  /* PHYLIP requires trifurcation */
  /* memory allocation */
  r = utreec[(numsp-3)*4]; l = utreec[1+(numsp-3)*4];
  len = strlen(tstring[r]) + strlen(tstring[l]) + 16 +
    ndig_intpart(utreec[2+(numsp-3)*4])+ndig_intpart(utreec[3+(numsp-3)*4]);
  if (r >= numsp) len += strlen(ilabels[r-numsp]);
  if (l >= numsp) len += strlen(ilabels[l-numsp]);
  tstring[2*numsp-3] = (char *) malloc((size_t) len*sizeof(char));
  /* create subtree */
  i=numsp-3;
  if (r >= numsp && l >= numsp)
    alen=sprintf(tstring[i+numsp],"%s%s:%.5f,%s%s:%.5f",tstring[r],
	    ilabels[r-numsp],utreec[2+i*4],tstring[l],ilabels[l-numsp],
	    utreec[3+i*4]);
  if (r >= numsp && l < numsp)
    alen=sprintf(tstring[i+numsp],"%s%s:%.5f,%s:%.5f",tstring[r],
	    ilabels[r-numsp],utreec[2+i*4],tstring[l],utreec[3+i*4]);
  if (r < numsp && l >= numsp)
    alen=sprintf(tstring[i+numsp],"%s:%.5f,%s%s:%.5f",tstring[r],
	    utreec[2+i*4],tstring[l],ilabels[l-numsp],utreec[3+i*4]);
  if (r < numsp && l < numsp)
    alen=sprintf(tstring[i+numsp],"%s:%.5f,%s:%.5f",tstring[r],
	    utreec[2+i*4],tstring[l],utreec[3+i*4]);
  /* printf("%i %i %i %i %i: %s\n",r,l,len,alen+1,2*numsp-3,tstring[2*numsp-3]); */
  /* memory allocation for final string */
  r = utreec[(numsp-2)*4]; l = utreec[1+(numsp-2)*4];
  r = (r < l)?r:l;
  x=utreec[2+(numsp-2)*4]+utreec[3+(numsp-2)*4];
  len = strlen(tstring[2*numsp-3]) + strlen(tstring[r]) + 12 + 
    ndig_intpart(x);
  if (r >= numsp) len += strlen(ilabels[r-numsp]);
  tstr=(char *) malloc((size_t) len*sizeof(char));
  /* tree */
  if (r >= numsp)
    alen=sprintf(tstr,"(%s,%s%s:%.5f);",tstring[2*numsp-3],tstring[r],
	    ilabels[r-numsp],x);
  if (r < numsp)
    alen=sprintf(tstr,"(%s,%s:%.5f);",tstring[2*numsp-3],tstring[r],x);
  tstr[len-1] = '\0';
  /* printf("%i %i %i %s\n",r,len,alen+1,tstr); */
  for(i = 0; i < 2*numsp-2; i++) free(tstring[i]);
  free(tstring);
  return(tstr);
}
void pr_tstr_labels(FILE *otreefile, int ntaxa, double *utreec, 
		    char names[][11]){
  char **ilabels,*tstr,(*namest)[11];
  int i,j,elab,len;

  ilabels = (char **)malloc((size_t) (ntaxa-2)*sizeof(char *));
  namest=(char (*)[11]) malloc((size_t) ntaxa*sizeof(*namest));
  for(i=0; i<ntaxa; i++) 
    for(j=0; j<11; j++) namest[i][j]=names[i][j];
  for(i = 0; i < ntaxa-2; i++){
    elab=i+ntaxa;
    len=1;
    while(elab>=exp(len*log(10))) len++;
    ilabels[i] = (char *)malloc((size_t) (len+1)*sizeof(char));
    sprintf(ilabels[i],"%i",elab);
  }
  tstr=ut2in(ntaxa,utreec,namest,ilabels);
  fprintf(otreefile,"%s\n", tstr);

  for(i=0; i<ntaxa-2; i++) free(ilabels[i]);
  free(ilabels);
  free(tstr); free(namest);
}
void split_leading1(int numsp, int *split){
  int i;
  if (split[0] == 1) return;
  for(i = 0; i < numsp; i++) split[i] = (split[i] == 1)?0:1;
}
void utreec2splits(int numsp, double *utreec, int *splits){
  int i,r,l,split;
  /* ith column gives split for numsp+ith branch */
  for(split = 0; split < numsp-3; split++){
    r = (int) utreec[split*4]; l = (int) utreec[1+split*4];
    if (r < numsp){
      for(i = 0; i < numsp; i++) splits[i+split*numsp] = 0;
      splits[r+split*numsp] = 1;
    }
    else
      for(i = 0; i < numsp; i++) splits[i+split*numsp] = splits[i+(r-numsp)*numsp];
    if (l < numsp)
      splits[l+split*numsp] = 1;
    else
      for(i = 0; i < numsp; i++) splits[i+split*numsp] += splits[i+(l-numsp)*numsp];
  }
  for(split = 0; split < numsp-3; split++)
    split_leading1(numsp,&splits[split*numsp]);
}
int nrow_ftaxa_file(FILE *infile){
  int i,nrow;
  nrow=0;
  while(fscanf(infile,"%i",&i)>0) nrow++;
  if(nrow % 5 != 0){
    printf("file should have 5 entries per row\n");
    exit(1);
  } 
  nrow /= 5;
  rewind(infile);
  return(nrow);
}
int check_split_ftaxa(int *split, int *ftaxat){
  if(split[ftaxat[0]]==split[ftaxat[1]] &&
     split[ftaxat[2]]==split[ftaxat[3]] &&
     split[ftaxat[0]]!=split[ftaxat[2]])
    return(1);
  else
    return(0);
}
void utreec2du(int ntaxa, double *utreec, int *du){
  int i,k,j,c;

  for(i=0; i<ntaxa*(ntaxa-1); i++) du[i]=0.0;

  for(j=0; j<ntaxa-1; j++) 
    for(k=0; k<2; k++){
      c=utreec[k+j*4];
      if(c<ntaxa) 
	du[c+j*ntaxa]=k+1;
      else
	for(i=0; i<ntaxa; i++)
	  if(du[i+(c-ntaxa)*ntaxa]>0) du[i+j*ntaxa]=k+1;
    }
  return;
}
void du2fgp(int ntaxa, double *utreec, int *du){
  int p,k,i,c,s;
 
  for(p=0; p<ntaxa-2; p++) 
    for(k=0; k<2; k++)
      if((c=utreec[k+p*4]) >= ntaxa){
	s=(k==0)?utreec[1+p*4]:utreec[p*4];
	if(s<ntaxa)
	  du[s+(c-ntaxa)*ntaxa]=3;
	else
	  for(i=0; i<ntaxa; i++)
	    if(du[i+(s-ntaxa)*ntaxa]==1 || du[i+(s-ntaxa)*ntaxa]==2) 
	      du[i+(c-ntaxa)*ntaxa]=3;
      }
  if((c=utreec[(ntaxa-2)*4]) >= ntaxa)
    for(i=0; i<ntaxa; i++){
      if(du[i+(ntaxa-3)*ntaxa]==1) du[i+(c-ntaxa)*ntaxa]=0;
      if(du[i+(ntaxa-3)*ntaxa]==2) du[i+(c-ntaxa)*ntaxa]=3;
    }
  return;
}
void reorder_ftaxa(int i, int j, int k, int l, int *t){
  if(i>j){
    t[0]=i;
    i=j;
    j=t[0];
  }
  if(k>l){
    t[0]=k;
    k=l;
    l=t[0];
  }
  if(i<k){
    t[0]=i;t[1]=j;t[2]=k;t[3]=l;
  }
  else{
    t[0]=k;t[1]=l;t[2]=i;t[3]=j;
  }
  return;
}
int du2ftaxa(int ntaxa, double *utreec, int *du, int *ftaxa){
  int nft,e,p0,p1,p2,p3;

  nft=0;
  for(e=0; e<ntaxa-3; e++)
    for(p0=0; p0<ntaxa; p0++)
      if(du[p0+e*ntaxa]==0)
	for(p1=0; p1<ntaxa; p1++)
	  if(du[p1+e*ntaxa]==1)
	    for(p2=0; p2<ntaxa; p2++)
	      if(du[p2+e*ntaxa]==2)
		for(p3=0; p3<ntaxa; p3++)
		  if(du[p3+e*ntaxa]==3){
		    reorder_ftaxa(p1,p2, p0,p3, &ftaxa[nft*6]);
		    ftaxa[4+nft*6]=e+ntaxa;
		    nft++;
		  }
  return(nft);
}
int boot4taxa(FILE *treefile, int ntaxa, char names[][11], int B, int *ftaxa){
  int b,i,is_found,nft,*ftc,nftc,*du,j;
  double *utreec;
  
  utreec = (double *) malloc(4*(ntaxa-1)*sizeof(double));
  nftc = (ntaxa*(ntaxa-1)*(ntaxa-2)*(ntaxa-3))/24;
  ftc = (int *) malloc(6*nftc*sizeof(int));
  du=(int *) malloc((ntaxa-1)*ntaxa*sizeof(int));
  
  nft=0;
  for(b=0; b<B; b++){
    read_treecsnl(treefile,ntaxa,names,utreec,1);
    /* if(b==245) pr_utreec(stdout,ntaxa,utreec); */
    utreec2du(ntaxa,utreec,du);
    du2fgp(ntaxa,utreec,du);
    nftc=du2ftaxa(ntaxa,utreec,du,ftc);
    for(i=0; i<nftc; i++){
      is_found=0;
      if(ftc[0+i*6]==0 && ftc[1+i*6]==24 && ftc[2+i*6]==4 && ftc[3+i*6]==6) printf("%i %i\n",b, ftc[4+i*6]);
      for(j=0; j<nft; j++)
	if(isamev(4,&ftc[i*6],1,&ftaxa[j*5],1)){
	  ftaxa[4+j*5]++;
	  is_found=1;
	  break;
	}
      if(!is_found){
	memcpy(&ftaxa[nft*5],&ftc[i*6],4*sizeof(int));
	ftaxa[4+nft*5]=1;
	nft++;
      }
    }
  }
  exit(0);
  free(utreec);
  free(ftc);
  free(du);
  return(nft);
}
void bpftaxa(int ntaxa, int nftb, int *ftaxab, int nft, int *ftaxat){
  int i,j;
  
  for(i=0; i<nft; i++){
    ftaxat[5+6*i]=0;
    for(j=0; j<nftb; j++)
      if(isamev(4,&ftaxat[6*i],1,&ftaxab[5*j],1)){
	ftaxat[5+6*i]=ftaxab[4+5*j];
	break;
      }
  }
  return;
}
int lstring(char * string){
  /* returns length of string */
  int l = 0;
  while (string[l] != '\0') l++;
  return(l);
}
int isamev(int n, int *a, int inca, int *b, int incb){

  while(n-- > 0){
    if(*a!=*b) return(0);
    a += inca; b+=incb;
  }
  
  return(1);
}
